/* Implement keyboard functions to manage multiple keys presses without
repet delays.

This class is used for X Window (Linux) event handling.

Every time one key is pressed (program receive a key press message) one key position -
in key table - is set TRUE, when is released (program receive a key release message)
is set FALSE.
*/



#include "defs.h"



BOOL ayKEYS [ 256 ];    // key table: TRUE: given key is pressed else FALSE



#include <string.h>

#define XK_MISCELLANY 1
#include <X11/keysymdef.h>

#include "keycodes.h"
#include "keyboard.h"



/* constructor
**********************************************************/
CKeyboard::CKeyboard ()
{
   // reset key table (all FALSE)
   memset ( (void*) ayKEYS, FALSE, sizeof ( ayKEYS ) );
}



/* destructor
**********************************************************/
CKeyboard::~CKeyboard ()
{
}



/* Set a specific key in key table acording the given
 key pressed/released in X Window event. This function
is used to translate key codes between X and PC BIOS codes.
dwXKeyCode - the X Window key code
yValue - value to set the key
**********************************************************/
void CKeyboard::mSetKeyTable ( DWORD dwXKeyCode, BYTE yValue )
{
   BYTE
      yKey = 0;   // receive the PC BIOS key code
      
   // make translation
   switch ( dwXKeyCode ) {
   
      case XK_BackSpace:
         yKey = K_BACKSPACE;
         break;

      case XK_Tab:
         yKey = K_TAB;
         break;
   
      case XK_Return:
         yKey = K_ENTER;
         break;
   
      case XK_Escape:
         yKey = K_ESC;
         break;
   
      case XK_Home:
         yKey = K_HOME;
         break;
   
      case XK_End:
         yKey = K_END;
         break;
   
      case XK_Left:
         yKey = K_LEFT;
         break;
   
      case XK_Right:
         yKey = K_RIGHT;
         break;
   
      case XK_Up:
         yKey = K_UP;
         break;
   
      case XK_Down:
         yKey = K_DOWN;
         break;
   
      case XK_Page_Up:
         yKey = K_PG_UP;
         break;
   
      case XK_Page_Down:
         yKey = K_PG_DN;
         break;

   
      case XK_KP_Enter:
         yKey = K_ENTER + SPECIAL;
         break;

      case XK_KP_Home:
         yKey = K_HOME + SPECIAL;
         break;
   
      case XK_KP_End:
         yKey = K_END + SPECIAL;
         break;
   
      case XK_KP_Left:
         yKey = K_LEFT + SPECIAL;
         break;
   
      case XK_KP_Right:
         yKey = K_RIGHT + SPECIAL;
         break;
   
      case XK_KP_Up:
         yKey = K_UP + SPECIAL;
         break;
   
      case XK_KP_Down:
         yKey = K_DOWN + SPECIAL;
         break;
   
      case XK_KP_Page_Up:
         yKey = K_PG_UP + SPECIAL;
         break;
   
      case XK_KP_Page_Down:
         yKey = K_PG_DN + SPECIAL;
         break;
   

      /* REVIEW who are this X keys?
      case XK_KP_0:
         yKey = K_0 + SPECIAL;
         break;

      case XK_KP_1:
         yKey = K_1 + SPECIAL;
         break;

      case XK_KP_2:
         yKey = K_2 + SPECIAL;
         break;

      case XK_KP_3:
         yKey = K_3 + SPECIAL;
         break;

      case XK_KP_4:
         yKey = K_4 + SPECIAL;
         break;

      case XK_KP_5:
         yKey = K_5 + SPECIAL;
         break;

      case XK_KP_6:
         yKey = K_6 + SPECIAL;
         break;

      case XK_KP_7:
         yKey = K_7 + SPECIAL;
         break;

      case XK_KP_8:
         yKey = K_8 + SPECIAL;
         break;

      case XK_KP_9:
         yKey = K_9 + SPECIAL;
         break;
      */

      case 65457:
         yKey = K_END; // + SPECIAL;
         break;
      
      case 65458:
         yKey = K_DOWN + SPECIAL;
         break;
      
      case 65459:
         yKey = K_PG_DN; // + SPECIAL;
         break;
      
      case 65460:
         yKey = K_LEFT + SPECIAL;
         break;
      
      case 65461:
         yKey = K_5 + SPECIAL;  // REVIEW who is midle key 5???
         break;
      
      case 65462:
         yKey = K_RIGHT + SPECIAL;
         break;

      case 65463:
         yKey = K_HOME; // + SPECIAL;
         break;
      
      case 65464:
         yKey = K_UP + SPECIAL;
         break;
      
      case 65465:
         yKey = K_PG_UP; // + SPECIAL;
         break;
      
   
      case XK_F1:
         yKey = K_F1;
         break;
   
      case XK_F2:
         yKey = K_F2;
         break;

      case XK_F3:
         yKey = K_F3;
         break;

      case XK_F4:
         yKey = K_F4;
         break;

      case XK_F5:
         yKey = K_F5;
         break;

      case XK_F6:
         yKey = K_F6;
         break;

      case XK_F7:
         yKey = K_F7;
         break;

      case XK_F8:
         yKey = K_F8;
         break;

      case XK_F9:
         yKey = K_F9;
         break;

      case XK_F10:
         yKey = K_F10;
         break;

      case XK_F11:
         yKey = K_F11;
         break;

      case XK_F12:
         yKey = K_F12;
         break;


      case XK_Shift_L:
         yKey = K_SHIFT_L;
         break;

      case XK_Shift_R:
         yKey = K_SHIFT_R;
         break;

      case XK_Control_L:
         yKey = K_CTRL;
         break;

      case XK_Control_R:
         yKey = K_CTRL + SPECIAL;
         break;

      case XK_Alt_L:
         yKey = K_ALT;
         break;
         
      case XK_Alt_R:
         yKey = K_ALT + SPECIAL;
         break;
         

      case 97:
         yKey = K_A;
         break;

      case 98:
         yKey = K_B;
         break;

      case 99:
         yKey = K_C;
         break;

      case 100:
         yKey = K_D;
         break;

      case 101:
         yKey = K_E;
         break;

      case 102:
         yKey = K_F;
         break;

      case 103:
         yKey = K_G;
         break;

      case 104:
         yKey = K_K;
         break;

      case 105:
         yKey = K_I;
         break;

      case 106:
         yKey = K_J;
         break;

      case 107:
         yKey = K_K;
         break;

      case 108:
         yKey = K_L;
         break;

      case 109:
         yKey = K_M;
         break;

      case 110:
         yKey = K_N;
         break;

      case 111:
         yKey = K_O;
         break;

      case 112:
         yKey = K_P;
         break;

      case 113:
         yKey = K_Q;
         break;

      case 114:
         yKey = K_R;
         break;

      case 115:
         yKey = K_S;
         break;

      case 116:
         yKey = K_T;
         break;

      case 117:
         yKey = K_U;
         break;

      case 118:
         yKey = K_V;
         break;

      case 119:
         yKey = K_W;
         break;
         
      case 120:
         yKey = K_X;
         break;

      case 121:
         yKey = K_Y;
         break;

      case 122:
         yKey = K_Z;
         break;

      default:        // if key conversion not found
         yKey = (BYTE) dwXKeyCode;  // very suspect, one last try...
         break;
   }
         
   // set key table
   * ( ayKEYS + yKey ) = yValue;
}



/* Verify if a specific key is pressed now.
 return TRUE - yes; FALSE - no
**********************************************************/
BOOL CKeyboard::mTestKeyPress ( BYTE yKey )
{
   return * ( ayKEYS + yKey );
}



/* Return the code of the first key pressed found.
If no key pressed return 0.
**********************************************************/
BYTE CKeyboard::mTestAllPress ()
{
   // see all keys's table
   for ( unsigned short int ucT = 0; ucT < 256; ucT ++ )
      // if find TRUE return it's code
      if ( mTestKeyPress ( ucT ) == TRUE )
         return ucT;

   return 0;
}



/* Wait some key be pressed. If bTestPress == TRUE then
only wait key pressed after all keys be released.
Default bTestPress = TRUE. return key code.
**********************************************************/
BYTE CKeyboard::mWaitAllPress ( BOOL bTestPress )
{
   // verify if must wait all keys be realeased
   if ( bTestPress ) {
      BOOL
         bRelease;
      // while one key be pressed
      do {
         bRelease = TRUE;
         // see all key's table
         for ( unsigned short int ucT = 0; ucT < 256; ucT ++ )
            // if find one pressed
            if ( mTestKeyPress ( ucT ) == TRUE )
               bRelease = FALSE;
      } while ( bRelease == FALSE );
   }

   // while any key not pressed
   while ( 1 )
      // see all key's table
      for ( unsigned short int ucT = 0; ucT < 256; ucT ++ )
         // if find one pressed return his code
         if ( mTestKeyPress ( ucT ) == TRUE )
            return ucT;
}



/* Wait all key be released
**********************************************************/
void CKeyboard::mWaitAllRelease ()
{
   BOOL
      bRelease;

   // while any key be pressed
   do {

      bRelease = TRUE;
      // see key's table
      for ( unsigned short int ucT = 0; ucT < 256; ucT ++ )
         // if find any key pressed
         if ( mTestKeyPress ( ucT ) == TRUE )
            bRelease = FALSE;

   } while ( bRelease == FALSE );
}

