/* How to set use the X Window to show a frame buffer, set palette and make keyboard input.

Distribution under GNU-GPL.
Autor : Leandro Bernsmuller 
e-mail: lebe@geocities.com
site  : http://www.geocities.com/SiliconValley/Bay/1704

To compile just type make, see README.
*/



#include <string.h>     // memset
#include <stdlib.h>     
#include <stdio.h>      

// need for X:
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <errno.h>
#define XK_MISCELLANY 1
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#include <X11/keysymdef.h>

#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Box.h>

#ifdef COMP_SHAREMEMORY
#include <X11/extensions/XShm.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#endif

#include "defs.h"
#include "keycodes.h"
#include "keyboard.h"
#include "xwinapp.h"



// Declare a X Window app instance
CXWindowApp
   oAPP;



/* This is the user handler - run one main loop interaction.
**********************************************************/
void UserHandler ( XtPointer stPos, XtIntervalId * pstInt )
{
   static unsigned int
      dwFrames = 0;
   
   // verify if exist frame buffer (exit when window message Expose is received)
   if ( oAPP.m_pyVid ) {

      // scan user frame buffer slowly
      oAPP.m_pyVidPal [ dwFrames % ( 600 * 400 ) ] = dwFrames % 256;
 
      // move pixel with keyboard input
      
      static int
         nX = 300,   // pixel coordenates
         nY = 225;

      // change coordenates acording the pressed keys
      if ( oAPP.m_oKb.mTestKeyPress ( K_LEFT ) )
         nX = ( nX <= 0 ) ? nX : nX - 1;
      if ( oAPP.m_oKb.mTestKeyPress ( K_RIGHT ) )
         nX = ( nX >= 599 ) ? nX : nX + 1;
      if ( oAPP.m_oKb.mTestKeyPress ( K_UP ) )
         nY = ( nY <= 0 ) ? nY : nY - 1;
      if ( oAPP.m_oKb.mTestKeyPress ( K_DOWN ) )
        nY = ( nY >= 449 ) ? nY : nY + 1;

      // set a pixel on user frame buffer
      memset ( (void*) ( oAPP.m_pyVidPal + 600 * nY + nX ), 1, 1 );

      // show user frame buffer
      oAPP.mShow ();
      
      dwFrames ++;
   }

   XtAppAddTimeOut ( oAPP.m_stApContext, 1, UserHandler, NULL );
}



int main ( int nNumArgs, char *szArgs [] )
{
   printf ( "\nGraphics under X Window System.\n\n" );
   printf ( "Demo - how to open one window, set palette, draw on window and make keyboard input." );
   printf ( "\n\nUnder demo, try press cursor arrow keys (UP, DOWN, RIGHT, LEFT) to move point (has diagonal).\nESC exit." );
   printf ( "\n\nWhen the window is open, put mouse pointer inside it." );    
   
   #ifdef COMP_SHAREMEMORY
   // edit the makefile to enable/disable share memory
   printf ( "\nShare memory enabled." );
   #endif
   #ifndef COMP_SHAREMEMORY
   printf ( "\nShare memory disabled." );
   #endif

   printf ( "\n\nPress ENTER..." );    
   getchar ();

   // init window app
   oAPP.mCreate ( "Demo - ESC: exit; Cursor arrow keys: move", 600, 450, nNumArgs, szArgs );
   // oAPP is a global instance for the X app interface, see xwinapp.cc

   // set palette
   BYTE
       ayPal [ 256 * 3 ];
   // all other colors
   for ( int ncP = 0; ncP < 256 * 3; ncP += 3 ) {
      ayPal [ ncP + 0 ] = 60;
      ayPal [ ncP + 1 ] = 255 - ncP;
      ayPal [ ncP + 2 ] = 60;
   }
   // background
   ayPal [ 0 ] = 255;
   ayPal [ 1 ] = 255;
   ayPal [ 2 ] = 255;
   // foreground
   ayPal [ 3 ] = 0;
   ayPal [ 4 ] = 0;
   ayPal [ 5 ] = 0;
   oAPP.mWritePal ( ayPal );

   // execute system loop
   XtAppAddTimeOut ( oAPP.m_stApContext, 300, UserHandler, NULL );
   XtAppMainLoop ( oAPP.m_stApContext );
}

